//****************************************************************************
// sRTC_Time.h
// Set, Read, and Display Time
// PORTx Defined in Main Routine
// Pin F4  = SDA,  Pin F3 = SCL
// Time Set similar to digital watch.
// Enter PB - Changes Mode and returns to Time Clock
// UP PB - Increments number
// This routine includes Day of Week and Month Text Displays
// 04.07.2009
//****************************************************************************
unsigned char const monthname[] = {
         ' ', ' ', ' ',
         'J', 'A', 'N',
         'F', 'E', 'B',
         'M', 'A', 'R',
         'A', 'P', 'R',
         'M', 'A', 'Y',
         'J', 'U', 'N',
         'J', 'U', 'L',
         'A', 'U', 'G',
         'S', 'E', 'P',
         'O', 'C', 'T',
         'N', 'O', 'V',
         'D', 'E', 'C' };
unsigned char const dayname[] = {
         ' ', ' ', ' ',
         'M', 'O', 'N',
         'T', 'U', 'E',
         'W', 'E', 'D',
         'T', 'H', 'U',
         'F', 'R', 'I',
         'S', 'A', 'T',
         'S', 'U', 'N' };
//****************************************************************************
struct TTime {
  int seconds, minutes, hours, day, date, month, year, control, twentyfour;
} TimeRead;
char SecondsLast;
//****************************************************************************
//****************************************************************************
void Delay200ms()
{
Delay_ms(200);
}
//****************************************************************************
void Get_Time()
{
// Read DS1307
Soft_I2C_Start();
Soft_I2C_Write(0b11010000);                // Write Mode
Soft_I2C_Write(0);
Delay_us(50);
Soft_I2C_Start();
Soft_I2C_Write(0b11010001);                // Read Mode
TimeRead.seconds = Soft_I2C_Read(1);
TimeRead.minutes = Soft_I2C_Read(1);
TimeRead.hours   = Soft_I2C_Read(1);
TimeRead.day     = Soft_I2C_Read(1);
TimeRead.date    = Soft_I2C_Read(1);
TimeRead.month   = Soft_I2C_Read(1);
TimeRead.year    = Soft_I2C_Read(1);
TimeRead.control = Soft_I2C_Read(0);       // No Acknowledge if 0
Soft_I2C_Stop();
TimeRead.twentyfour = TimeRead.hours;
     if (TimeRead.hours == Dec2BCD(0))
        {
        TimeRead.hours = Dec2BCD(12);
        }
     if (TimeRead.twentyfour >= Dec2BCD(13))
        {
        TimeRead.hours = (BCD2Dec(TimeRead.twentyfour) - 12);
        }
     if (TimeRead.twentyfour >= Dec2BCD(22))
        {
        TimeRead.hours = (BCD2Dec(TimeRead.twentyfour) - 6);
        }
     if (TimeRead.hours > Dec2BCD(23))
        {
        TimeRead.hours = 1;
        }
}
//****************************************************************************
void Display_Time_Short(unsigned char row, unsigned char col)
{
char i, txt1[12];

    if (TimeRead.seconds != SecondsLast)
       {
       txt1[2] = ':';
       }
    else
       txt1[2] = ' ';

   txt1[0] = (TimeRead.hours >> 4) + 48;
   txt1[1] = (TimeRead.hours & 0x0F) + 48;
//   txt1[2] = ':';
   txt1[3] = (TimeRead.minutes >> 4) + 48;
   txt1[4] = (TimeRead.minutes & 0x0F) + 48;
   txt1[5] = ' ';
   if (TimeRead.twentyfour > Dec2BCD(11))
     txt1[6] = 'P';
   else
     txt1[6] = 'A';
   txt1[7] = 'M';
   txt1[8] = 0;
   sLCD_Chr(row, col, txt1[0]);
   for (i = 1; i < 8; i++)
      {
      sLCD_Chr_Cp(txt1[i]);
      }
SecondsLast = TimeRead.seconds;
}
//****************************************************************************
void Display_Day(unsigned char row, unsigned char col)
{
char i, txt[11];
   for (i = 0; i < 3; i++)
      {
      txt[i] = dayname[((BCD2Dec(Timeread.day) * 3) + i)];
      }

   sLCD_Chr(row, col, txt[0]);
   for (i = 1; i < 3; i++)
      {
      sLCD_Chr_Cp(txt[i]);
      }
}
//****************************************************************************
void Display_Date(unsigned char row, unsigned char col)
{
char i, txt[11];
   for (i = 0; i < 3; i++)
      {
      txt[i] = monthname[((BCD2Dec(Timeread.month) * 3) + i)];
      }
//   txt[0] = (TimeRead.month >> 4) + 48;
//   txt[1] = (TimeRead.month & 0x0F) + 48;
   txt[3] = '/';
   txt[4] = (TimeRead.date >> 4) + 48;
   txt[5] = (TimeRead.date & 0x0F) + 48;
   txt[6] = '/';
   txt[7] = (TimeRead.year >> 4) + 48;
   txt[8] = (TimeRead.year & 0x0F)  + 48;
   txt[9] = 0;
   sLCD_Chr(row, col, txt[0]);
   for (i = 1; i < 9; i++)
      {
      sLCD_Chr_Cp(txt[i]);
      }
}
//****************************************************************************
void Set_Time()
{
char i, j, n, TempVal[7], txt[4], dateloop;
   j = -1;
   n = 0;
   dateloop = 0;
   Get_Time();
   TempVal[0] = BCD2Dec(TimeRead.twentyfour);
   TempVal[1] = BCD2Dec(TimeRead.minutes);
   TempVal[2] = BCD2Dec(TimeRead.seconds);
   TempVal[3] = BCD2Dec(TimeRead.month);
   TempVal[4] = BCD2Dec(TimeRead.date);
   TempVal[5] = BCD2Dec(TimeRead.year);
   TempVal[6] = BCD2Dec(TimeRead.day);
   sLCD_Out(sLCD_SECOND_ROW, 1, "                ");
   sLCD_Out(sLCD_FIRST_ROW, 1, "Set Time     ");
   sLCD_Cmd(sLCD_UNDERLINE_ON);
   
while (1 == 1)
   {
//*******************************************
   if (dateloop == 2)
   {
   for (i = 0; i < 3; i++)
      {
      txt[i] = dayname[((TempVal[6] * 3) + i)];
      }

   sLCD_Chr(sLCD_SECOND_ROW, 8, txt[0]);
   for (i = 1; i < 3; i++)
      {
      sLCD_Chr_Cp(txt[i]);
      }
   }
//*******************************************
   if (dateloop == 1)
   {
   for (i = 0; i < 3; i++)
      {
      txt[i] = monthname[((TempVal[3] * 3) + i)];
      }

   sLCD_Chr(sLCD_SECOND_ROW, 8, txt[0]);
   for (i = 1; i < 3; i++)
      {
      sLCD_Chr_Cp(txt[i]);
      }
   }
//********************************************
   if (dateloop == 0)
   {
   txt[0] = (Dec2BCD(TempVal[j]) >> 4) + 48;
   txt[1] = (Dec2BCD(TempVal[j]) & 0x0F) + 48;
   txt[2] = ' ';
   sLCD_Chr(sLCD_SECOND_ROW, 8, txt[0]);
   for (i = 1; i < 3; i++)
      {
      sLCD_Chr_Cp(txt[i]);
      }
   }
//********************************************
   if (EN == 0)
      {
      while (EN == 0)
      {
      }
      ++n;
      }
//********************************************
   if (UP == 0)
      {
      while (UP == 0)
        {
        }
        ++TempVal[j];
      }
      
switch(n)
{
//***************************************************************************
case 0:   //HOURS   Set Hours in 24 Hour Time

  dateloop = 0;
  j = 0;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Hours: ");
  if (TempVal[j] >= 24)
     TempVal[j] = 0;

break;
//***************************************************************************
case 1:   //MINUTES

  j = 1;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Mins : ");
  if (TempVal[j] >= 60)
     TempVal[j] = 0;


break;
//***************************************************************************
case 2:   //SECONDS

  j = 2;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Secs : ");
  if (TempVal[j] >= 60)
     TempVal[j] = 0;


break;
//***************************************************************************
case 3:   //MONTHS

  dateloop = 1;
  j = 3;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Month: ");
  if (TempVal[j] >= 13)
     TempVal[j] = 1;


break;
//***************************************************************************
case 4:    //DATE

  dateloop = 0;
  j = 4;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Date : ");
  if (TempVal[j] >= 32)
     TempVal[j] = 0;


break;
//***************************************************************************
case 5:    //YEAR

  j = 5;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Year : ");
  if (TempVal[j] >= 16)
     TempVal[j] = 8;


break;
//***************************************************************************
case 6:    //DAY

  dateloop = 2;
  j = 6;
  sLCD_Out(sLCD_SECOND_ROW, 1, "Day : ");
  if (TempVal[j] >= 8)
     TempVal[j] = 1;


break;
//***************************************************************************
case 7:

     sResetLCD();
     TimeRead.seconds = Dec2BCD(TempVal[2]);
     TimeRead.minutes = Dec2BCD(TempVal[1]);
     TimeRead.hours   = Dec2BCD(TempVal[0]);
     TimeRead.day     = Dec2BCD(TempVal[6]);
     TimeRead.date    = Dec2BCD(TempVal[4]);
     TimeRead.month   = Dec2BCD(TempVal[3]);
     TimeRead.year    = Dec2BCD(TempVal[5]);
     TimeRead.control = 0;
     Soft_I2C_Start();
     Soft_I2C_Write(0b11010000);          // Write Mode
     Soft_I2C_Write(0);                   // Start with First Address
     Soft_I2C_Write(TimeRead.seconds);
     Soft_I2C_Write(TimeRead.minutes);
     Soft_I2C_Write(TimeRead.hours);
     Soft_I2C_Write(TimeRead.day);
     Soft_I2C_Write(TimeRead.date);
     Soft_I2C_Write(TimeRead.month);
     Soft_I2C_Write(TimeRead.year);
     Soft_I2C_Write(TimeRead.control);
     Soft_I2C_Stop();
     return;

break;

Delay200ms();
    } // End Switch
  } // End While
} // End Set_Time
//****************************************************************************
Alarm(char hh, char mm, char ss, char mo, char dd, char yr)
{
char trip;
Get_Time();
if (TimeRead.twentyfour == Dec2BCD(hh) && TimeRead.minutes == Dec2BCD(mm)
   && TimeRead.seconds == Dec2BCD(ss) && TimeRead.month == Dec2BCD(mo)
    && TimeRead.date == Dec2BCD(dd) && TimeRead.year == Dec2BCD(yr))
   {
   trip = 1;
   return trip;
   }
   trip = 0;
   return trip;
}
//****************************************************************************